# Generated file: frontend_assets.py

DIR__APP__FILE__PAGE__TSX = '''// src/app/page.tsx
import React from 'react';
import {{ redirect }} from 'next/navigation';

const HomePage: React.FC = () => {{

  const modals = process.env.MODALS.split(',');

  // Perform server-side redirection
  redirect(`/${{modals[0]}}`);

  return null; // No need to return any JSX as the redirection happens immediately
}};

export default HomePage;'''

DIR__APP__FILE__LAYOUT__TSX = '''import type {{ Metadata }} from "next";
import {{ Inter }} from "next/font/google";
import "./globals.css";

const inter = Inter({{ subsets: ["latin"] }});

export const metadata: Metadata = {{
  title: "Create Next App",
  description: "Generated by create next app",
}};

export default function RootLayout({{
  children,
}}: Readonly<{{
  children: React.ReactNode;
}}>) {{
  return (
    <html lang="en">
      <body className={{inter.className}}>{{children}}</body>
    </html>
  );
}}'''

DIR__APP__FILE__GLOBALS__CSS = '''@tailwind base;
@tailwind components;
@tailwind utilities;'''

DIR__COMPONENTS__FILE__CREATE_MODAL__TSX = '''import React, {{ useState, useEffect }} from 'react';

interface CreateModalProps {{
  modalName: string;
  apiHost: string;
  columns: string[];
  onClose: () => void;
}}

const CreateModal: React.FC<CreateModalProps> = ({{ modalName, apiHost, columns, onClose }}) => {{
  const [formData, setFormData] = useState<{{ [key: string]: any }}>({{}});
  const [userId, setUserId] = useState<string | null>(null);

  useEffect(() => {{
    // Extract user_id from cookies
    const cookies = document.cookie.split(';').reduce((acc, cookie) => {{
      const [key, value] = cookie.trim().split('=');
      acc[key] = value;
      return acc;
    }}, {{}} as {{ [key: string]: string }});

    setUserId(cookies.user_id || null);
  }}, []);

  const handleChange = (e: React.ChangeEvent<HTMLInputElement>) => {{
    setFormData({{
      ...formData,
      [e.target.name]: e.target.value
    }});
  }};

  const handleSubmit = async () => {{
    try {{
      const dataToSubmit = {{ ...formData, user_id: userId }};
      console.log('35',dataToSubmit);
      const response = await fetch(`${{apiHost}}create/${{modalName}}`, {{
        method: 'POST',
        headers: {{
          'Content-Type': 'application/json'
        }},
        body: JSON.stringify(dataToSubmit)
      }});
      if (response.ok) {{
        alert('Record created successfully');
        onClose();
	 window.location.reload();
      }} else {{
        alert('Failed to create record');
      }}
    }} catch (error) {{
      console.error('Error creating record:', error);
    }}
  }};

  const filteredColumns = columns.filter(column => !['id', 'created_at', 'updated_at', 'user_id'].includes(column));

  return (
    <div className="fixed inset-0 bg-black bg-opacity-75 flex items-center justify-center z-50">
      <div className="bg-gray-800 p-8 rounded-lg shadow-lg text-white w-96">
        <h2 className="text-2xl mb-4">Create New {{modalName.charAt(0).toUpperCase() + modalName.slice(1)}}</h2>
        {{filteredColumns.map((column) => (
          <input
            key={{column}}
            type="text"
            name={{column}}
            placeholder={{column}}
            onChange={{handleChange}}
            className="bg-gray-700 p-2 rounded mb-4 w-full"
          />
        ))}}
        <div className="flex justify-end">
          <button onClick={{onClose}} className="bg-red-500 hover:bg-red-700 text-white py-2 px-4 rounded mr-2">Cancel</button>
          <button onClick={{handleSubmit}} className="bg-green-500 hover:bg-green-700 text-white py-2 px-4 rounded">Create</button>
        </div>
      </div>
    </div>
  );
}};

export default CreateModal;'''

DIR__COMPONENTS__FILE__SIDEBAR__TSX = '''import React from 'react';
import Link from 'next/link';
import {{ useRouter }} from 'next/router';

const Sidebar: React.FC = () => {{
  const modals = ['customers', 'partners'];
  const router = useRouter();
  const {{ modal }} = router.query;

  const handleLogout = () => {{
    document.cookie = 'user_id=; Max-Age=0; path=/';
    document.cookie = 'auth=; Max-Age=0; path=/';
    router.push('/login');
  }};

  return (
    <div className="bg-gray-800 text-gray-100 w-64 min-h-screen p-4 flex flex-col justify-between">
      <div>
        <h2 className="text-2xl font-semibold mb-6">Menu</h2>
        <ul>
          {{modals.map((item) => (
            <li key={{item}} className={{`mb-4 p-2 rounded ${{modal === item ? 'bg-gray-700' : 'hover:bg-gray-700 transition duration-300'}}`}}>
              <Link href={{`/${{item}}`}}>
                <span className="text-white cursor-pointer">{{item.charAt(0).toUpperCase() + item.slice(1)}}</span>
              </Link>
            </li>
          ))}}
        </ul>
      </div>
      <button
        onClick={{handleLogout}}
        className="bg-red-500 hover:bg-red-700 text-white py-2 px-4 rounded focus:outline-none focus:ring-2 focus:ring-red-500"
      >
        Logout
      </button>
    </div>
  );
}};

export default Sidebar;'''

DIR__COMPONENTS__FILE__DYNAMIC_TABLE__TSX = '''import React, {{ useState, useEffect }} from 'react';
import CreateModal from './CreateModal';
import EditModal from './EditModal';

interface DynamicTableProps {{
  apiHost: string;
  modal: string;
  columns: string[];
  data: any[];
}}

const DynamicTable: React.FC<DynamicTableProps> = ({{ apiHost, modal, columns, data: initialData }}) => {{
  const [data, setData] = useState<any[]>(initialData);
  const [isCreateModalOpen, setCreateModalOpen] = useState(false);
  const [isEditModalOpen, setEditModalOpen] = useState(false);
  const [editRowData, setEditRowData] = useState<{{ [key: string]: any }} | null>(null);
  const [filterQuery, setFilterQuery] = useState(''); // State for the filter query

  useEffect(() => {{
    setData(initialData);
  }}, [initialData]);

  useEffect(() => {{
    if (!initialData.length) {{
      const fetchData = async () => {{
        try {{
          const response = await fetch(`${{apiHost}}read/${{modal}}`);
          const result = await response.json();
          setData(result.data || []);
        }} catch (error) {{
          console.error('Error fetching data:', error);
          setData([]);
        }}
      }};
      fetchData();
    }}
  }}, [apiHost, modal, initialData]);

  const handleDelete = async (id: number, userId: number) => {{
    try {{
      const response = await fetch(`${{apiHost}}delete/${{modal}}/${{id}}`, {{
        method: 'DELETE',
        headers: {{
          'Content-Type': 'application/json',
        }},
        body: JSON.stringify({{ user_id: userId }}),
      }});
      const result = await response.json();
      if (result.status === 'success') {{
        setData(data.filter((row: any) => row[0] !== id)); // Assuming the first column is the ID
      }}
    }} catch (error) {{
      console.error('Error deleting data:', error);
    }}
  }};

  const handleEdit = (row: {{ [key: string]: any }}) => {{
    setEditRowData(row);
    setEditModalOpen(true);
  }};

  const handleCreate = () => {{
    setCreateModalOpen(true);
  }};

  const closeCreateModal = () => setCreateModalOpen(false);

  const closeEditModal = (updatedData: any[] | null) => {{
    if (updatedData) {{
      // Update the rowData with the new data
      setData(prevData => prevData.map(row => row[0] === updatedData[0] ? updatedData : row));
    }}
    setEditModalOpen(false);
  }};

  const handleFilterChange = (event: React.ChangeEvent<HTMLInputElement>) => {{
    setFilterQuery(event.target.value);
  }};

  const evaluateFilter = (row: any, query: string): boolean => {{
    const regex = /(\w+)\s*(CONTAINS|STARTS_WITH|==|>|<|>=|<=)\s*'([^']*)'/i;
    const match = query.match(regex);

    if (!match) return true;

    const [, column, operator, value] = match;
    const columnIndex = columns.indexOf(column);

    if (columnIndex === -1) return false;

    const cellValue = row[columnIndex];

    switch (operator) {{
      case 'CONTAINS':
        return cellValue.toString().toLowerCase().includes(value.toLowerCase());
      case 'STARTS_WITH':
        return cellValue.toString().toLowerCase().startsWith(value.toLowerCase());
      case '==':
        return cellValue.toString() === value;
      case '>':
        return new Date(cellValue) > new Date(value) || parseFloat(cellValue) > parseFloat(value);
      case '<':
        return new Date(cellValue) < new Date(value) || parseFloat(cellValue) < parseFloat(value);
      case '>=':
        return new Date(cellValue) >= new Date(value) || parseFloat(cellValue) >= parseFloat(value);
      case '<=':
        return new Date(cellValue) <= new Date(value) || parseFloat(cellValue) <= parseFloat(value);
      default:
        return true;
    }}
  }};

  const filteredData = data.filter(row => evaluateFilter(row, filterQuery));

  const formatDateTime = (dateTime: string) => {{
    const date = new Date(dateTime);
    const year = date.getFullYear();
    const month = ('0' + (date.getMonth() + 1)).slice(-2);
    const day = ('0' + date.getDate()).slice(-2);
    const hours = ('0' + date.getHours()).slice(-2);
    const minutes = ('0' + date.getMinutes()).slice(-2);
    const seconds = ('0' + date.getSeconds()).slice(-2);
    return `${{year}}-${{month}}-${{day}} ${{hours}}:${{minutes}}:${{seconds}}`;
  }};

  return (
    <div className="bg-gray-900 text-white p-4">
      <div className="flex justify-between mb-4">
        <input
          type="text"
          value={{filterQuery}}
          onChange={{handleFilterChange}}
          placeholder="Filter [==, >, >=, <, <=, CONTAINS, STARTS_WITH] ..."
          className="bg-gray-800 text-white px-4 py-2 rounded w-1/3"
        />
        <button
          onClick={{handleCreate}}
          className="bg-blue-500 hover:bg-blue-700 text-white font-bold py-2 px-4 rounded"
        >
          Create
        </button>
      </div>
      <div className="overflow-x-auto">
        <table className="min-w-full divide-y divide-gray-700">
          <thead>
            <tr>
              {{columns.map((col, colIndex) => (
                <th
                  key={{`col-${{colIndex}}`}}
                  className="px-3 py-3 text-left text-xs font-medium text-gray-300 tracking-wider"
                >
                  {{col}}
                </th>
              ))}}
              <th className="px-3 py-3 text-left text-xs font-medium text-gray-300 uppercase tracking-wider">Actions</th>
            </tr>
          </thead>
          <tbody>
            {{filteredData.map((row, rowIndex) => (
              <tr key={{`row-${{rowIndex}}`}} className="bg-gray-800">
                {{row.map((cell: any, cellIndex: number) => (
                  <td
                    key={{`cell-${{rowIndex}}-${{cellIndex}}`}}
                    className="px-3 py-4 whitespace-nowrap text-sm text-gray-300"
                  >
                    {{typeof cell === 'string' && !isNaN(Date.parse(cell)) ? formatDateTime(cell) : cell}}
                  </td>
                ))}}
                <td className="px-3 py-4 whitespace-nowrap text-sm text-gray-300">
                  <button
                    onClick={{() => handleEdit(row)}}
                    className="bg-yellow-500 hover:bg-yellow-700 text-white font-bold py-1 px-2 rounded mr-2"
                  >
                    Edit
                  </button>
                  <button
                    onClick={{() => handleDelete(row[0], row.user_id)}} // Pass the user_id here
                    className="bg-red-500 hover:bg-red-700 text-white font-bold py-1 px-2 rounded"
                  >
                    Delete
                  </button>
                </td>
              </tr>
            ))}}
          </tbody>
        </table>
      </div>
      {{isCreateModalOpen && <CreateModal modalName={{modal}} apiHost={{apiHost}} columns={{columns}} onClose={{closeCreateModal}} />}}
      {{isEditModalOpen && editRowData && <EditModal modalName={{modal}} apiHost={{apiHost}} columns={{columns}} rowData={{editRowData}} onClose={{closeEditModal}} />}}
    </div>
  );
}};

export default DynamicTable;'''

DIR__COMPONENTS__FILE__EDIT_MODAL__TSX = '''import React, {{ useState, useEffect }} from 'react';

interface EditModalProps {{
  modalName: string;
  apiHost: string;
  columns: string[];
  rowData: any[];
  onClose: (updatedData: any[]) => void; // Modify onClose to accept updated data
}}

const EditModal: React.FC<EditModalProps> = ({{ modalName, apiHost, columns, rowData, onClose }}) => {{
  const [formData, setFormData] = useState<{{ [key: string]: any }}>({{}});

  useEffect(() => {{
    // Convert rowData to an object with column names as keys
    const initialFormData = columns.reduce((acc, column, index) => {{
      acc[column] = rowData[index];
      return acc;
    }}, {{}} as {{ [key: string]: any }});
    setFormData(initialFormData);
  }}, [columns, rowData]);

  const handleChange = (e: React.ChangeEvent<HTMLInputElement>) => {{
    setFormData({{
      ...formData,
      [e.target.name]: e.target.value
    }});
  }};

  const handleSubmit = async () => {{
    try {{
      const response = await fetch(`${{apiHost}}update/${{modalName}}/${{formData.id}}`, {{
        method: 'PUT',
        headers: {{
          'Content-Type': 'application/json'
        }},
        body: JSON.stringify(formData)
      }});
      if (response.ok) {{
        alert('Record updated successfully');
        onClose(Object.values(formData)); // Pass updated data to onClose
      }} else {{
        alert('Failed to update record');
      }}
    }} catch (error) {{
      console.error('Error updating record:', error);
    }}
  }};

  return (
    <div className="fixed inset-0 bg-black bg-opacity-75 flex items-center justify-center z-50">
      <div className="bg-gray-800 p-8 rounded-lg shadow-lg text-white w-96">
        <h2 className="text-2xl mb-4">
          Edit {{modalName.charAt(0).toUpperCase() + modalName.slice(1)}} ({{rowData[0]}})
        </h2>
        {{columns.map((column) => (
          column !== 'created_at' && column !== 'updated_at' && column !== 'id' && column !== 'user_id' && (
            <div key={{column}} className="mb-4">
              <label className="block text-sm font-medium text-gray-300">{{column}}</label>
              <input
                type="text"
                name={{column}}
                value={{formData[column] || ''}}
                placeholder={{column}}
                onChange={{handleChange}}
                className="bg-gray-700 p-2 rounded w-full"
              />
            </div>
          )
        ))}}
        <div className="flex justify-end">
          <button onClick={{() => onClose(null)}} className="bg-red-500 hover:bg-red-700 text-white py-2 px-4 rounded mr-2">Cancel</button>
          <button onClick={{handleSubmit}} className="bg-green-500 hover:bg-green-700 text-white py-2 px-4 rounded">Update</button>
        </div>
      </div>
    </div>
  );
}};

export default EditModal;'''

DIR__PAGES__FILE__LOGIN__TSX = '''import React, {{ useState }} from 'react';
import {{ useRouter }} from 'next/navigation';
import '../app/globals.css';

const isAuthenticated = () => {{
  const cookies = document.cookie.split(';').reduce((acc, cookie) => {{
    const [key, value] = cookie.trim().split('=');
    acc[key] = value;
    return acc;
  }}, {{}});

  return cookies.user_id !== undefined;
}};

const LoginPage: React.FC = () => {{
  const [username, setUsername] = useState('');
  const [password, setPassword] = useState('');
  const [error, setError] = useState('');
  const router = useRouter();
  const apiHost = process.env.NEXT_PUBLIC_API_HOST;

  const handleSubmit = async (e: React.FormEvent) => {{
    e.preventDefault();

    if (isAuthenticated()) {{
      router.push('/');
      return;
    }}

    try {{
      const response = await fetch(`${{apiHost}}/authenticate`, {{
        method: 'POST',
        headers: {{
          'Content-Type': 'application/json',
        }},
        body: JSON.stringify({{ username, password }}),
      }});

      const result = await response.json();
      console.log(result); // Debugging: Log the result
      if (result.status === 'success') {{
        // Set cookie for authentication
        const user = result.user;
        document.cookie = `user_id=${{user.id}}; path=/`;
        document.cookie = `auth=true; path=/`;
        console.log("Authentication successful, redirecting..."); // Debugging: Log before redirect
        router.push('/');
      }} else {{
        setError(result.message);
      }}
    }} catch (error) {{
      console.error("Error during authentication:", error);
      setError("An error occurred during authentication. Please try again.");
    }}
  }};

  return (
    <div className="min-h-screen bg-gray-900 text-white flex items-center justify-center">
      <div className="bg-gray-800 p-8 rounded-lg shadow-lg w-96">
        <h2 className="text-2xl mb-4 text-center font-semibold">Login</h2>
        <form onSubmit={{handleSubmit}}>
          <div className="mb-4">
            <label className="block text-sm font-medium text-gray-300">Username</label>
            <input
              type="text"
              name="username"
              value={{username}}
              onChange={{(e) => setUsername(e.target.value)}}
              className="bg-gray-700 p-2 rounded w-full focus:outline-none focus:ring-2 focus:ring-green-500"
              required
            />
          </div>
          <div className="mb-4">
            <label className="block text-sm font-medium text-gray-300">Password</label>
            <input
              type="password"
              name="password"
              value={{password}}
              onChange={{(e) => setPassword(e.target.value)}}
              className="bg-gray-700 p-2 rounded w-full focus:outline-none focus:ring-2 focus:ring-green-500"
              required
            />
          </div>
          {{error && <p className="text-red-500 mb-4">{{error}}</p>}}
          <div className="flex justify-end">
            <button
              type="submit"
              className="bg-green-500 hover:bg-green-700 text-white py-2 px-4 rounded focus:outline-none focus:ring-2 focus:ring-green-500"
            >
              Login
            </button>
          </div>
        </form>
      </div>
    </div>
  );
}};

export default LoginPage;'''

DIR__PAGES__FILE__PAGE__TSX = ''''''

DIR__PAGES__FILE____MODAL____TSX = '''import React, {{ useEffect, useState }} from 'react';
import {{ useRouter }} from 'next/router';
import DynamicTable from '../components/DynamicTable';
import Sidebar from '../components/Sidebar';
import '../app/globals.css';

const ModalPage: React.FC = () => {{
  const router = useRouter();
  const {{ modal }} = router.query;

  const [data, setData] = useState<any[]>([]);
  const [columns, setColumns] = useState<string[]>([]);

  const apiHost = process.env.NEXT_PUBLIC_API_HOST;

  useEffect(() => {{
    console.log("ModalPage useEffect called");
    console.log("modal:", modal);
    console.log("apiHost:", apiHost);

    if (modal && apiHost) {{
      const fetchData = async () => {{
        try {{
          console.log(`Fetching data from ${{apiHost}}read/${{modal}}`);
          const response = await fetch(`${{apiHost}}read/${{modal}}`);
          console.log("Response received:", response);

          if (response.ok) {{
            const fetchedData = await response.json();
            console.log("Fetched data:", fetchedData);
            setColumns(fetchedData.columns);
            setData(fetchedData.data);
          }} else {{
            console.error("Error response:", response.statusText);
            setData([]);
            setColumns([]);
          }}
        }} catch (error) {{
          console.error(`Error fetching data for ${{modal}}:`, error);
          setData([]);
          setColumns([]);
        }}
      }};

      fetchData();
    }}
  }}, [modal, apiHost]);

  if (!modal) {{
    return <div>Loading...</div>;
  }}

  return (
    <div className="flex">
      <Sidebar />
      <div className="bg-black min-h-screen flex-1 p-8">
        <div className="container mx-auto">
          <h1 className="text-4xl text-white font-bold mb-4">{{(modal as string)?.charAt(0).toUpperCase() + (modal as string)?.slice(1)}} Management</h1>
          <DynamicTable apiHost={{apiHost}} modal={{modal as string}} columns={{columns}} data={{data}} />
        </div>
      </div>
    </div>
  );
}};

export default ModalPage;'''

ROOT__FILE__MIDDLEWARE__TSX = '''import {{ NextResponse }} from 'next/server';
import type {{ NextRequest }} from 'next/server';

export function middleware(request: NextRequest) {{
  const user_id = request.cookies.get('user_id');

  if (!user_id && request.nextUrl.pathname !== '/login') {{
    return NextResponse.redirect(new URL('/login', request.url));
  }}

  if (user_id && request.nextUrl.pathname === '/login') {{
    return NextResponse.redirect(new URL('/', request.url));
  }}

  return NextResponse.next();
}}

const modals = process.env.MODALS.split(',')

export const config = {{


  matcher: ['/', '/customers', '/partners', '/login'],
}};'''




import os
import json
from .resources.backend import main as backend_main
from .resources.frontend import main as frontend_main

class f:
    def __init__(self):
        pass

    def ser(self, db_preset_name, new_db_name, vm_preset_name, modal_map, backend_deploy_at, backend_deploy_port, frontend_deploy_path):
        """[f.ser(db_preset_name='your_rgwml_config_mysql_preset_name', new_db_name='name_of_db_to_br_created', vm_preset_name='your_rgwml_config_gcs_vm_preset_name',modal_map={'customers': 'mobile,issue,status', 'partners': 'mobile,issue,status'}, backend_deploy_at='path/on/your/vm/to/deploy/your/backend', backend_deploy_port='8080', frontend_deploy_path='/path/on/your/local/machine/to/provision/your/frontend')]"""
        def locate_config_file(filename="rgwml.config"):
            home_dir = os.path.expanduser("~")
            search_paths = [
                os.path.join(home_dir, "Desktop"),
                os.path.join(home_dir, "Documents"),
                os.path.join(home_dir, "Downloads"),
            ]

            for path in search_paths:
                for root, dirs, files in os.walk(path):
                    if filename in files:
                        return os.path.join(root, filename)
            raise FileNotFoundError(f"{filename} not found in Desktop, Documents, or Downloads folders")

        def load_db_config(db_preset_name):
            config_path = locate_config_file()
            with open(config_path, 'r') as f:
                config = json.load(f)

            db_presets = config.get('db_presets', [])
            db_preset = next((preset for preset in db_presets if preset['name'] == db_preset_name), None)
            if not db_preset:
                raise ValueError(f"No matching db_preset found for {db_preset_name}")

            return db_preset

        def load_vm_config(vm_preset_name):
            config_path = locate_config_file()
            with open(config_path, 'r') as f:
                config = json.load(f)

            vm_presets = config.get('vm_presets', [])
            vm_preset = next((preset for preset in vm_presets if preset['name'] == vm_preset_name), None)
            if not vm_preset:
                raise ValueError(f"No matching vm_preset found for {vm_preset_name}")

            return vm_preset

        # Load DB config
        db_preset = load_db_config(db_preset_name)
        db_type = db_preset['db_type']

        if db_type == 'mysql':
            host = db_preset['host']
            username = db_preset['username']
            password = db_preset['password']
            database = db_preset.get('database', '')

            db_config = {
                'host': host,
                'user': username,
                'password': password,
                'database': new_db_name
            }

        # Load VM config
        if vm_preset_name:
            vm_preset = load_vm_config(vm_preset_name)
            ssh_key_path = vm_preset['ssh_key_path']
            gcs_instance = vm_preset['gcs_instance']
            vm_host = vm_preset['host']
        else:
            raise ValueError("vm_preset_name is required")

        # Infer modals from modal_map keys
        modals = ','.join(modal_map.keys())

        # Deploy backend
        backend_main(db_config, modal_map, ssh_key_path, gcs_instance, backend_deploy_at, backend_deploy_port)

        # Deploy frontend
        frontend_main(frontend_deploy_path, vm_host, backend_deploy_port, modals)





import subprocess
import os
import shutil

# Import the generated frontend assets
from .frontend_assets import *

def create_nextjs_project(project_path, use_src):
    src_flag = '--src-dir' if use_src else ''
    subprocess.run(['npx', 'create-next-app@latest', project_path, '--typescript', '--eslint', '--tailwind', '--app','--no-import-alias', src_flag], check=True)
    os.chdir(project_path)
    subprocess.run(['npm', 'install', '-D', 'tailwindcss', 'postcss', 'autoprefixer'], check=True)
    subprocess.run(['npx', 'tailwindcss', 'init', '-p'], check=True)

def configure_tailwind(use_src):
    content_paths = [
        "./app/**/*.{js,ts,jsx,tsx,mdx}",
        "./pages/**/*.{js,ts,jsx,tsx,mdx}",
        "./components/**/*.{js,ts,jsx,tsx,mdx}"
    ]
    if use_src:
        content_paths.append("./src/**/*.{js,ts,jsx,tsx,mdx}")
    tailwind_config = f"""
/** @type {{import('tailwindcss').Config}} */
module.exports = {{
  content: {content_paths},
  theme: {{
    extend: {{}},
  }},
  plugins: [],
}}
"""
    with open('tailwind.config.js', 'w') as f:
        f.write(tailwind_config)

    globals_css = """
@tailwind base;
@tailwind components;
@tailwind utilities;
"""
    styles_dir = 'src/styles' if use_src else 'styles'
    os.makedirs(styles_dir, exist_ok=True)
    with open(os.path.join(styles_dir, 'globals.css'), 'w') as f:
        f.write(globals_css)

def remove_conflicting_files(project_path):
    if os.path.exists(project_path):
        shutil.rmtree(project_path)
    os.makedirs(project_path, exist_ok=True)

def remove_src_directory(project_path):
    src_dir = os.path.join(project_path, 'src')
    if os.path.exists(src_dir):
        shutil.rmtree(src_dir)  # Remove the entire src directory

def create_env_file(project_path, api_host, modals):
    env_content = f"""
NEXT_PUBLIC_API_HOST={api_host}
MODALS={modals}
"""
    with open(os.path.join(project_path, '.env'), 'w') as f:
        f.write(env_content)

def to_camel_case(snake_str):
    components = snake_str.split('_')
    return components[0].lower() + ''.join(x.title() for x in components[1:])

def main(frontend_deploy_path,host,api_port,modals):
    use_src = True

    project_path = frontend_deploy_path
    api_host = f"http://{host}:{api_port}/"
    remove_conflicting_files(project_path)
    create_nextjs_project(project_path, use_src)
    remove_src_directory(project_path)  # Remove the src directory after creating the project
    configure_tailwind(use_src)
    create_env_file(project_path, api_host, modals)

    # Recreate the src directory and necessary subdirectories
    src_dir = os.path.join(project_path, 'src')
    os.makedirs(src_dir, exist_ok=True)
    os.makedirs(os.path.join(src_dir, 'pages'), exist_ok=True)
    os.makedirs(os.path.join(src_dir, 'components'), exist_ok=True)

    # Dictionary mapping variable names to file paths
    file_mapping = {
        "DIR__APP__FILE__PAGE__TSX": os.path.join(src_dir, "app/page.tsx"),
        "DIR__APP__FILE__LAYOUT__TSX": os.path.join(src_dir, "app/layout.tsx"),
        "DIR__APP__FILE__GLOBALS__CSS": os.path.join(src_dir, "app/globals.css"),
        "DIR__COMPONENTS__FILE__CREATE_MODAL__TSX": os.path.join(src_dir, "components/CreateModal.tsx"),
        "DIR__COMPONENTS__FILE__EDIT_MODAL__TSX": os.path.join(src_dir, "components/EditModal.tsx"),
        "DIR__COMPONENTS__FILE__SIDEBAR__TSX": os.path.join(src_dir, "components/Sidebar.tsx"),
        "DIR__COMPONENTS__FILE__DYNAMIC_TABLE__TSX": os.path.join(src_dir, "components/DynamicTable.tsx"),
        "DIR__PAGES__FILE__PAGE__TSX": os.path.join(src_dir, "pages/page.tsx"),
        "DIR__PAGES__FILE____MODAL____TSX": os.path.join(src_dir, "pages/[modal].tsx"),
        "DIR__PAGES__FILE__LOGIN__TSX": os.path.join(src_dir, "pages/login.tsx"),
        "ROOT__FILE__MIDDLEWARE__TSX": os.path.join(src_dir, "pages/middleware.tsx"),
        # Add other mappings as needed
    }

    # Create necessary files from frontend assets
    for var_name, file_path in file_mapping.items():
        if var_name in globals():
            content = globals()[var_name]
            # Replace {{ and }} with { and }
            content = content.replace('{{', '{').replace('}}', '}')
            dir_path = os.path.dirname(file_path)
            os.makedirs(dir_path, exist_ok=True)
            with open(file_path, 'w') as f:
                f.write(content)

    subprocess.run(['npm', 'run', 'dev'], check=True)




import pymysql
import subprocess
import os
import time
import json

def create_database(config):
    temp_config = config.copy()
    temp_config.pop('database', None)  # Remove the database key to connect to the server

    conn = pymysql.connect(**temp_config)
    cursor = conn.cursor()
    cursor.execute(f"CREATE DATABASE IF NOT EXISTS {config['database']}")
    cursor.close()
    conn.close()

def create_user_table(config):
    conn = pymysql.connect(**config)
    cursor = conn.cursor()
    cursor.execute("""
    CREATE TABLE IF NOT EXISTS users (
        id INT AUTO_INCREMENT PRIMARY KEY,
        username TEXT,
        password TEXT,
        created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
        updated_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP
    )
    """)
    cursor.close()
    conn.close()

def create_modal_table(config, modal_name, columns):
    conn = pymysql.connect(**config)
    cursor = conn.cursor()
    columns_definition = ", ".join([f"{col} TEXT" for col in columns])
    cursor.execute(f"""
    CREATE TABLE IF NOT EXISTS {modal_name} (
        id INT AUTO_INCREMENT PRIMARY KEY,
        user_id INT,
        {columns_definition},
        created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
        updated_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP
    )
    """)
    cursor.execute(f"""
    CREATE TABLE IF NOT EXISTS {modal_name}_logs (
        id INT AUTO_INCREMENT PRIMARY KEY,
        user_id INT,
        operation_type ENUM('CREATE', 'UPDATE', 'DELETE'),
        operation_details JSON,
        created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
        updated_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP
    )
    """)
    cursor.close()
    conn.close()

def create_bottle_app(modal_map, config, deploy_port):
    app_code = f"""
import pymysql
import json
from bottle import Bottle, request, response, run, error, HTTPResponse
from datetime import datetime

config = {config}

modal_map = {modal_map}

app = Bottle()

@app.hook('before_request')
def enable_cors():
    response.headers['Access-Control-Allow-Origin'] = '*'
    response.headers['Access-Control-Allow-Methods'] = 'GET, POST, PUT, DELETE, OPTIONS'
    response.headers['Access-Control-Allow-Headers'] = 'Origin, Accept, Content-Type, X-Requested-With, X-CSRF-Token'
    if request.method == 'OPTIONS':
        return HTTPResponse(status=200)

def serialize_datetime(obj):
    if isinstance(obj, datetime):
        return obj.isoformat()
    raise TypeError("Type not serializable")

@app.error(500)
def error500(error):
    return "Internal Server Error"

@app.route('/favicon.ico')
def serve_favicon():
    return {{"status":"success"}}

@app.route('/authenticate', method=['POST', 'OPTIONS'])
def authenticate():
    if request.method == 'OPTIONS':
        return {{}}

    data = request.json
    username = data.get('username')
    password = data.get('password')

    try:
        conn = pymysql.connect(**config)
        cursor = conn.cursor(pymysql.cursors.DictCursor)
        cursor.execute("SELECT id, username, password FROM users WHERE username = %s", (username,))
        user = cursor.fetchone()
        cursor.close()
        conn.close()

        if user and user['password'] == password:
            return {{"status": "success", "message": "Authentication successful", "user": user}}
        else:
            return {{"status": "error", "message": "Invalid username or password"}}
    except pymysql.MySQLError as e:
        return {{"status": "error", "message": str(e)}}

@app.route('/create/<modal_name>', method=['OPTIONS', 'POST'])
def create_entry(modal_name):
    if request.method == 'OPTIONS':
        return {{}}

    data = request.json
    columns = modal_map[modal_name].split(",")
    column_names = ", ".join(columns)
    placeholders = ", ".join(["%s"] * len(columns))
    values = [data.get(col) for col in columns]

    conn = pymysql.connect(**config)
    cursor = conn.cursor()
    cursor.execute(f"INSERT INTO {{modal_name}} (user_id, {{column_names}}) VALUES (%s, {{placeholders}})", [data['user_id']] + values)
    conn.commit()
    cursor.close()
    conn.close()

    log_operation(modal_name, data['user_id'], 'CREATE', {{"user_id": data['user_id'], **data}})
    return {{"status": "success"}}

@app.route('/read/<modal_name>', method=['OPTIONS', 'GET'])
def read_entries(modal_name):
    if request.method == 'OPTIONS':
        return {{}}

    conn = pymysql.connect(**config)
    cursor = conn.cursor()
    if modal_name != 'favicon':
        print(f"SELECT * FROM {{modal_name}}")
        cursor.execute(f"SELECT * FROM {{modal_name}}")
        result = cursor.fetchall()
        column_names = [desc[0] for desc in cursor.description]
        cursor.close()
        conn.close()

        # Serialize datetime objects
        serialized_result = []
        for row in result:
            serialized_row = []
            for item in row:
                if isinstance(item, datetime):
                    serialized_row.append(item.isoformat())
                else:
                    serialized_row.append(item)
            serialized_result.append(serialized_row)

        return {{"columns": column_names, "data": serialized_result}}

@app.route('/update/<modal_name>/<entry_id>', method=['PUT','OPTIONS'])
def update_entry(modal_name, entry_id):

    if request.method == 'OPTIONS':
        return {{}}

    data = request.json
    columns = modal_map[modal_name].split(",")
    
    # Remove columns that should not be updated
    columns = [col for col in columns if col not in ['id', 'created_at', 'updated_at', 'user_id']]

    # Construct the SET clause with placeholders for SQL parameters
    set_clause = ", ".join([f"{{col}} = %s" for col in columns])
    values = [data.get(col) for col in columns]

    conn = pymysql.connect(**config)
    cursor = conn.cursor()

    try:
        # Fetch the old row for logging purposes
        cursor.execute(f"SELECT * FROM {{modal_name}} WHERE id = %s", [entry_id])
        old_row = cursor.fetchone()
        print(old_row)

        # Perform the update operation
        cursor.execute(f"UPDATE {{modal_name}} SET {{set_clause}} WHERE id = %s", values + [entry_id])
        conn.commit()

        # Log the operation
        log_operation(modal_name, data['user_id'], 'UPDATE', {{"old": old_row, "new": data}})

        return {{"status": "success"}}
    except Exception as e:
        conn.rollback()
        return {{"status": "error", "message": str(e)}}
    finally:
        cursor.close()
        conn.close()

@app.route('/delete/<modal_name>/<entry_id>', method=['OPTIONS', 'DELETE'])
def delete_entry(modal_name, entry_id):

    if request.method == 'OPTIONS':
        return {{}}

    data = request.json
    user_id = data.get('user_id')
    conn = pymysql.connect(**config)
    cursor = conn.cursor()
    cursor.execute(f"SELECT * FROM {{modal_name}} WHERE id = %s", [entry_id])
    old_row = cursor.fetchone()
    cursor.execute(f"DELETE FROM {{modal_name}} WHERE id = %s", [entry_id])
    conn.commit()
    cursor.close()
    log_operation(modal_name, user_id, 'DELETE', {{ "deleted_row": old_row }})
    return {{"status": "success"}}

def log_operation(modal_name, user_id, operation_type, operation_details):
    conn = pymysql.connect(**config)
    cursor = conn.cursor()
    cursor.execute(f"INSERT INTO {{modal_name}}_logs (user_id, operation_type, operation_details) VALUES (%s, %s, %s)", [user_id, operation_type, json.dumps(operation_details, default=serialize_datetime)])
    conn.commit()
    cursor.close()
    conn.close()

run(app, host='0.0.0.0', port={deploy_port})
"""
    with open("app.py", "w") as f:
        f.write(app_code)

def kill_process_on_port(ssh_key_path, gcs_instance, port):
    kill_cmd = f"fuser -k {port}/tcp || true"
    subprocess.run(['ssh', '-i', ssh_key_path, gcs_instance, kill_cmd])

def deploy_to_gcs(ssh_key_path, gcs_instance, deploy_path):
    # Create the deployment directory on the remote server
    subprocess.run(['ssh', '-i', ssh_key_path, gcs_instance, f'mkdir -p {deploy_path}'])
    
    # Copy the app.py file to the remote server
    subprocess.run(['scp', '-i', ssh_key_path, 'app.py', f'{gcs_instance}:{deploy_path}/'])
    
    # Start the application on the remote server using nohup and detach from the SSH session
    subprocess.run(['ssh', '-i', ssh_key_path, gcs_instance, f'nohup bash -c "python3 {deploy_path}/app.py > {deploy_path}/app.log 2>&1 &"'])
    
    # Wait for the server to start
    time.sleep(5)

def run_tests(base_url, modal_map):
    for modal, columns in modal_map.items():
        columns_list = columns.split(',')
        test_data = {col: "xxxxx" for col in columns_list}
        test_data['user_id'] = 1

        print(f"\nTesting {modal} modal:\n")

        # Create entry
        create_response = subprocess.run([
            'curl', '-X', 'POST', f"{base_url}/create/{modal}",
            '-H', "Content-Type: application/json",
            '-d', json.dumps(test_data)
        ], capture_output=True, text=True)
        print("Create Entry Response:", create_response.stdout)

        # Read entries
        read_response = subprocess.run(['curl', '-X', 'GET', f"{base_url}/read/{modal}"], capture_output=True, text=True)
        print("Read Entries Response:", read_response.stdout)

        # Update entry
        updated_data = {col: "xxxxx_updated" for col in columns_list}
        updated_data['user_id'] = 1
        update_response = subprocess.run([
            'curl', '-X', 'PUT', f"{base_url}/update/{modal}/1",
            '-H', "Content-Type: application/json",
            '-d', json.dumps(updated_data)
        ], capture_output=True, text=True)
        print("Update Entry Response:", update_response.stdout)

        # Delete entry
        delete_response = subprocess.run([
            'curl', '-X', 'DELETE', f"{base_url}/delete/{modal}/1",
            '-H', "Content-Type: application/json",
            '-d', json.dumps({'user_id': 1})
        ], capture_output=True, text=True)
        print("Delete Entry Response:", delete_response.stdout)

def cleanup_db(config, modal_map):
    conn = pymysql.connect(**config)
    cursor = conn.cursor()
    for modal_name, columns in modal_map.items():
        columns_list = columns.split(',')
        conditions = " OR ".join([f"{col} LIKE '%%xxxxx%%'" for col in columns_list])
        
        # Delete from modal table
        query = f"DELETE FROM {modal_name} WHERE {conditions}"
        cursor.execute(query)
        
        # Delete from modal logs table
        log_conditions = " OR ".join([f"operation_details LIKE '%%{col}%%xxxxx%%'" for col in columns_list])
        log_query = f"DELETE FROM {modal_name}_logs WHERE {log_conditions}"
        cursor.execute(log_query)
        
        conn.commit()
    cursor.close()
    conn.close()


def main(config, modal_map, ssh_key_path, gcs_instance, deploy_at, deploy_port):
    create_database(config)
    create_user_table(config)

    for modal_name, columns in modal_map.items():
        create_modal_table(config, modal_name, columns.split(','))

    create_bottle_app(modal_map, config, deploy_port)

    # Ensure the port is cleared on the remote server
    kill_process_on_port(ssh_key_path, gcs_instance, deploy_port)

    deploy_to_gcs(ssh_key_path, gcs_instance, deploy_at)

    # Run dynamic tests
    run_tests(f"http://{config['host']}:{deploy_port}", modal_map)

    cleanup_db(config, modal_map)
    print(f"Backend deployed to: http://{config['host']}:{deploy_port}")
    


